@@ -17,8 +17,7 @@ use swc_ecma_utils::{
     private_ident, quote_ident, undefined, ExprFactory, ModuleItemLike, StmtLike,
 };
 use swc_ecma_visit::{
-    as_folder, noop_visit_mut_type, noop_visit_type, Fold, FoldWith, Visit, VisitMut, VisitMutWith,
-    VisitWith,
+    as_folder, noop_visit_mut_type, noop_visit_type, Fold, Visit, VisitMut, VisitMutWith, VisitWith,
 };
 
 mod class_name_tdz;
@@ -464,25 +463,22 @@ impl ClassProperties {
                         }
                     };
 
-                    let value = prop.value.unwrap_or_else(|| undefined(prop_span));
-                    let value = if prop.is_static {
-                        value
-                            .fold_with(&mut SuperFieldAccessFolder {
-                                class_name: &class_ident,
-                                vars: &mut vars,
-                                constructor_this_mark: None,
-                                is_static: true,
-                                folding_constructor: false,
-                                in_injected_define_property_call: false,
-                                in_nested_scope: false,
-                                this_alias_mark: None,
-                            })
-                            .fold_with(&mut as_folder(ThisInStaticFolder {
-                                ident: class_ident.clone(),
-                            }))
-                    } else {
-                        value
-                    };
+                    let mut value = prop.value.unwrap_or_else(|| undefined(prop_span));
+                    if prop.is_static {
+                        value.visit_mut_with(&mut SuperFieldAccessFolder {
+                            class_name: &class_ident,
+                            vars: &mut vars,
+                            constructor_this_mark: None,
+                            is_static: true,
+                            folding_constructor: false,
+                            in_injected_define_property_call: false,
+                            in_nested_scope: false,
+                            this_alias_mark: None,
+                        });
+                        value.visit_mut_with(&mut ThisInStaticFolder {
+                            ident: class_ident.clone(),
+                        });
+                    }
 
                     let callee = helper!(define_property, "defineProperty");
 
